@@ -188,8 +188,11 @@ func TestToDefinition(t *testing.T) {
 		},
 
 		"http_service": map[string]any{
-			"internal_port": int64(8080),
-			"force_https":   true,
+			"internal_port":        int64(8080),
+			"force_https":          true,
+			"auto_start_machines":  false,
+			"auto_stop_machines":   false,
+			"min_machines_running": int64(0),
 			"concurrency": map[string]any{
 				"type":       "donuts",
 				"hard_limit": int64(10),
@@ -239,6 +242,7 @@ func TestToDefinition(t *testing.T) {
 		"deploy": map[string]any{
 			"release_command": "release command",
 			"strategy":        "rolling-eyes",
+			"max_unavailable": 0.2,
 		},
 		"env": map[string]any{
 			"FOO": "BAR",
@@ -309,9 +313,12 @@ func TestToDefinition(t *testing.T) {
 		},
 		"services": []map[string]any{
 			{
-				"internal_port": int64(8081),
-				"protocol":      "tcp",
-				"processes":     []any{"app"},
+				"internal_port":        int64(8081),
+				"protocol":             "tcp",
+				"processes":            []any{"app"},
+				"auto_start_machines":  false,
+				"auto_stop_machines":   false,
+				"min_machines_running": int64(1),
 				"concurrency": map[string]any{
 					"type":       "requests",
 					"hard_limit": int64(22),
