@@ -66,6 +66,10 @@ func GetMachineGuest(ctx context.Context, guest *api.MachineGuest) (*api.Machine
 		guest.GPUKind = m
 	}
 
+	if IsSpecified(ctx, "host-dedication-id") {
+		guest.HostDedicationID = GetString(ctx, "host-dedication-id")
+	}
+
 	return guest, nil
 }
 
@@ -95,4 +99,8 @@ var VMSizeFlags = Set{
 		Aliases:     []string{"vm-gpukind"},
 		Hidden:      true,
 	},
+	String{
+		Name:        "host-dedication-id",
+		Description: "The dedication id of the reserved hosts for your organization (if any)",
+	},
 }
